/*
	decode arguments

	Copyright (C) 1991 K.Abe

	Patch for Human3.02 by Tachibana Eriko
*/


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <jctype.h>
#include "trace.h"
#include "syscall.inc"


static void decode_argument( int , void * , char * , System_call * , char * , int );
static void decode_argument_by_letter( char * , char * , void * );
static void escape( char * , char * );
static void fatchk( int , void * , char * );
static void   disk( int , void * , char * );


static System_call System_call_info[256];


extern void Initialize_argument_information( void )
{
    int	i , j;
    int	skip_len = strlen("v2_");

    for( i = 0 ; i < 256 ; i++ )
	System_call_info[i].name = NULL;

    for( i = 0 ; i < SYSTEM_CALL_TABLE_MAX ; i++ )
    {
	j = Human_system_call[i].number;

	System_call_info[ j ].name		= Human_system_call[ i ].name;
	System_call_info[ j ].argletter 	= Human_system_call[ i ].argletter;

	if( 0x50 <= j && j <= 0x7f )
	{
	System_call_info[ j+0x30 ].name		= Human_system_call[ i ].name + skip_len;
	System_call_info[ j+0x30 ].argletter	= Human_system_call[ i ].argletter;
	}
    }
}


#define DECODE(first,info_name)\
    decode_argument( *(first*)arg & 0xff, arg , argbuf ,\
		     info_name##_call_info ,\
		     System_call_info[ doscall ].name ,\
		     sizeof(info_name##_call_info)/sizeof(info_name##_call_info[0]) )

extern char *Format_output( int doscall , void *arg )
{
    static char argbuf[256];

    if( doscall < 0 || 256 < doscall || System_call_info[ doscall ].name == NULL )
    {
	sprintf( argbuf , "dos(%#x){UNDEFINED}" , doscall );
	return argbuf;
    }
    switch( doscall ) {
	case 0x0c:	/* kflush */
	    DECODE( short , Kflush );
	    break;
	case 0x17:	/* fatchk */
	    fatchk( doscall , arg , argbuf );
	    break;
	case 0x18:	/* hendsp */
	    DECODE( short , Hendsp );
	    break;
	case 0x44:	/* ioctrl */
	    DECODE( short , Ioctrl );
	    break;
	case 0x22:	/* knjctrl */
	    DECODE( long , Knjctrl );
	    break;
	case 0x23:	/* conctrl */
	    DECODE( short , Conctrl );
	    break;
	case 0x24:	/* keyctrl */
	    DECODE( short , Keyctrl );
	    break;
	case 0x4b:	/* exec */
	    DECODE( short , Exec );
	    break;
	case 0x55:	/* v2_common */
	    DECODE( short , v2Common );
	    break;
	case 0x5f:	/* v2_assign */
	    DECODE( short , v2Assign );
	    break;
	case 0x85:	/* common */
	    DECODE( short , Common );
	    break;
	case 0x8f:	/* assign */
	    DECODE( short , Assign );
	    break;
	case 0xf3:	/* diskred */
	case 0xf4:	/* diskwrt */
	    disk( doscall , arg , argbuf );
	    break;
	default:
	    decode_argument( doscall , arg , argbuf  , System_call_info , "dos" , 256 );
	    break;
    }
    return argbuf;
}


static void decode_argument( int call , void *arg , char *buffer , System_call *info , char *name ,int max )
{
    if( call >= max || info[ call ].name[0] == '\0' )
    {
	sprintf( buffer , "%s(%d){UNDEFINED}" , name , call );
	return;
    }
    strcpy( buffer , info[ call ].name );
    decode_argument_by_letter( buffer + strlen( buffer ) , info[ call ].argletter , arg );
}


static void decode_argument_by_letter( char *buffer , char *argletter , void *arg )
{
    char temp[256];
    *buffer++ = '(';

    while( *argletter )
    {
	switch( *argletter )
	{
	    case 'w': /* [hl */
		sprintf( buffer , "%d" , *(short*)arg );
		if( *(unsigned short*)arg >= 10 )
		{
		    buffer += strlen( buffer );
		    sprintf( buffer , "(%#x)" , *(short*)arg & 0xffff );
		}
		arg += 2;
		break;
	    case 'l': /* O[hl */
		sprintf( buffer , "%d" , *(long*)arg );
		if( *(long*)arg >= 10 )
		{
		    buffer += strlen( buffer );
		    sprintf( buffer , "(%#x)" , *(long*)arg );
		}
		arg += 4;
		break;
	    case 'p': /* |C^ */
		sprintf( buffer , "%#x" , *(long*)arg );
		arg += 4;
		break;
	    case 's': /*  */
		escape( (char *)*(long*)arg , temp );
		sprintf( buffer , "%s" , temp );
		arg += 4;
		break;
	    default:
		goto exit;
	}
	buffer += strlen( buffer );
	if( *++argletter )
	    *buffer++ = ',';
    }
exit:
    *buffer++ = ')';
    *buffer = '\0';
}


/* (܂̓AhX)\ */
static void escape( char *str , char *buffer )
{
    int	count;

    if( str >= (char*)0x01000000 )
    {
	sprintf( buffer , "(%#x)" , str );
	buffer += strlen( buffer );
	str = (char*)( (unsigned long)str & 0xffffff );
    }

    *buffer++ = '\"';
    for( count = 0 ; *str && count < 32 ; str++ , count++ )
    {
	if( isprkana( *str ) )
	    *buffer++ = *str;
	else if( iscntrl( *str ) )
	{
	    sprintf( buffer , "\\x%02x" , *str );
	    buffer += strlen( buffer );
	}
	else if( iskanji( *str ) && iskanji2( *( str + 1 ) ) )
	{
	    *buffer++ = *str++;
	    *buffer++ = *str;
	    count++;
	} else
	    *buffer++ = '.';
    }
    *buffer++ = '\"';

    if( *str )
    {
	*buffer++ = '.';
	*buffer++ = '.';
    }
    *buffer++ = '\0';
}


static void fatchk( int doscall , void *arg , char *argbuf )
{
    char *argletter;

    strcpy( argbuf ,  System_call_info[ doscall ].name );
    argbuf += strlen( argbuf );

    if( *(char*)(arg + 4) & 0x80 )
    {
	*argbuf++ = '{';
	*argbuf++ = '2';
	*argbuf++ = '}';
	argletter = "spw";
    }
    else
	argletter = "sp";

    decode_argument_by_letter( argbuf , argletter , arg );
}


static void disk( int doscall , void *arg , char *argbuf )
{
    char *argletter;

    strcpy( argbuf ,  System_call_info[ doscall ].name );
    argbuf += strlen( argbuf );

    if( *(char*)arg & 0x80 )
    {
	*argbuf++ = '{';
	*argbuf++ = '2';
	*argbuf++ = '}';
	argletter = "pwll";
    }
    else
	argletter = "pwww";

    decode_argument_by_letter( argbuf , argletter , arg );
}


/* EOF */
