/*
	System call tracer

	Copyright (C) 1991 K.Abe

	Patch for Human3.02 by Tachibana Eriko

*/

#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <sys/xunistd.h>
#include "trace.h"


static	 void	trace( const char* , const char** );
static	 void	convert_slash_to_backslash( char* );


FILE	*Stream = stdout;
int	Count = 0;
char	Option_A_flag = 0;

int main( int argc , char *argv[] )
{
    char	*command_name;
    char	**command_argv;
    char	*ofile = NULL;
    int		i;

    if( argc <= 1 )
	usage();

    for( i = 1 ; i < argc && argv[i][0] == '-' ; i++ )
    {
	switch( argv[i][1] )
	{
	    case 'o':
		if( argv[i][2] )
		    ofile = argv[i] + 2;
		else
		    ofile = argv[ ++i ];
		break;
	   case 'a':
		Option_A_flag = -1;
		if( argv[i][2] == 'o' )
		{
		    if( argv[i][3] )
			ofile = argv[i] + 3;
		    else
			ofile = argv[ ++i ];
		}
		break;
	   default:
		usage();
		break;
	}
    }
    if ( argc <= i )
	usage();

    check_human_version();

    if( ofile != NULL )
	if( ( Stream = fopen( ofile , "w" ) ) == NULL )
	{
	    perror( "fopen" );
	    exit(1);
	}

    if( _fddb[ fileno( Stream )].iomd & ( _DEV_CDEV | _DEV_RAW ) != _DEV_CDEV )
	fmode( Stream , _IOBINARY );

    Initialize_argument_information();
    command_name =  argv[i];
    command_argv = &argv[i];
    convert_slash_to_backslash( command_name );
    trace( command_name , (void *)command_argv );
    return 0;
}


static void trace( const char *command_name , const char *command_argv[] )
{
    int	rc;

    setup_doscall_handler();
    rc = spawnvp( P_WAIT , command_name , (void *)command_argv );
    restore_doscall_handler();

    if( rc == -1 )
    {
	perror( "spawnvp" );
	exit(1);
    }
    fprintf( Stream , "\nExit code %d (%d System Calls)\n" , rc , Count );
}

static void convert_slash_to_backslash( char *command_name )
{
    do
    {
	if( *command_name == '/')
	    *command_name = '\\';
    }
    while( *command_name++ );
}

/* EOF */
